# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/python-ftp-svn/source/MonitorExcel.py $
# Last committed:     $Revision: 197611 $; $Author: svcdms $; $Date: 2019-07-03 10:15:11 +0200 (Wed, 03 Jul 2019) $
#
import os
import glob

import pandas as pd
import numpy as np
import json
import sys
import getopt

import xlwings as xw

from datetime import datetime, timedelta

from .misc.Utils import Utils
from .dmsLogger.DmsLogger import DmsLogger

from hkvfewspy.utils.untangle import parse_raw
from hkvportal.io.services import dataportal


def get_active_sheet(workbook, sheet_name):

    active = None
    for sheet in workbook.sheets:
        if sheet_name in sheet.name:
            active = workbook.sheets[sheet_name]
            break

    if not active:
        # Creates a new Sheet and makes it the active sheet
        workbook.sheets.add(name=sheet_name)
        active = workbook.sheets[sheet_name]

    return active

def append_hyperlinks(workbook, hyperlinks):

    active = get_active_sheet(workbook, "Overzicht")
    active.range('A1:R2').api.Font.Bold = True

    active.range('A1').value = 'Overzicht'
    active.range('A2').value = 'Type'
    active.range('B2').value = 'Sensor'
    active.range('C2').value = 'Locatie'
    active.range('D2').value = 'Details'

    index = 3
    for hyperlink in hyperlinks:

        active.range('A'+str(index)).value = hyperlink['type']
        active.range('B'+str(index)).value = hyperlink['sensor']
        active.range('C'+str(index)).value = hyperlink['locatie']
        active.range('D'+str(index)).add_hyperlink(hyperlink['name'])

        index = index+1

    active.autofit()

def excel_data(settings):

    logger = DmsLogger.set_logger(
        settings["log_folder"], settings["logger"], settings["logfile"]
    )
    logger.info("Run MonitorExcel")

    excel_root = settings["excel_root"]

    excel_file = os.path.join(excel_root, settings["workbook"])
    if os.path.exists(excel_file):

        workbook = xw.Book(excel_file)

        files_in_folder = glob.glob("{}/*.csv".format(excel_root))

        hyperlinks = []
        for file in files_in_folder:
            logger.info("CSV file {}".format(file))

            df = pd.read_csv(file, index_col=0)
 
            type = 'ftp'
            if 'thredds_standard' in file:
                type = 'thredds_standard'
            elif 'thredds_tailored' in file:
                type = 'thredds_tailored'
  
            sensors = np.sort(df['Sensor'].unique())
            locaties = np.sort(df['Locatie'].unique())
            for sensor in sensors:
                if sensor != 'STA':
                    for locatie in locaties:

                        filter = df.loc[(df['Sensor'] == sensor) &
                                        (df['Locatie'] == locatie)]

                        if len(filter) > 0:
                            filter.index = np.arange(1, len(filter) + 1)
                            filter.index.names = ['Nummer']

                            sheet_name = "{}_{}_{}".format(type, sensor, locatie)
                            logger.info("Fill sheet {}".format(sheet_name))

                            active = get_active_sheet(workbook, sheet_name)

                            active.range('A1:R2').api.Font.Bold = True

                            active.range('E1').value = 'Gerealiseerd'
                            active.range('G2').value = 'Opmerking'
                            active.range('A2').value = filter
                            active.autofit()

                            hyperlink = {}
                            hyperlink['type'] = type
                            hyperlink['sensor'] = sensor
                            hyperlink['locatie'] = locatie
                            hyperlink['name'] = active.range('A1').get_address(row_absolute=True, column_absolute=True, include_sheetname=True, external=False)
                            
                            hyperlinks.append(hyperlink)

    append_hyperlinks(workbook, hyperlinks)                

    createHyperlinks = workbook.macro('CreateHyperlinks')
    createHyperlinks()

    workbook.save()
    #workbook.close()
    
    workbook.app.quit()

    return logger


def main(argv):
    # input argument checking
    try:
        opts, args = getopt.getopt(argv, "hs:", ["settings="])
    except getopt.GetoptError:
        print("MonitorExcel.py -s <settings_file>")
        sys.exit(2)
    for opt, arg in opts:
        if opt == "-h":
            print("MonitorExcel.py -s <settings_file>")
            sys.exit()
        elif opt in ("-s", "--settings"):
            settings_file = arg

    try:
        settings = Utils.read_excel_settings(settings_file)
        logger = excel_data(settings)
        logger.info("End of MonitorExcel")

    except Exception as e:
        print(e)
        exit(1)


if __name__ == "__main__":
    main(sys.argv[1:])
    # fire.Fire()
